var fs = require('fs');
var execFile = require('child_process').execFile;


/**
 * Walk a directory in a paralell loop.
 * http://stackoverflow.com/questions/5827612/node-js-fs-readdir-recursive-directory-search
 *
 * Usage:
 * > walk('./path/to/directory', callbackFunction);
 *
 */
function walk(dir, done) {
    var results = [];
    fs.readdir(dir, function(err, list) {
        if (err) return done(err);
        var pending = list.length;
        if (!pending) return done(null, results);
        list.forEach(function(file) {
            file = dir + '/' + file;
            fs.stat(file, function(err, stat) {
                if (stat && stat.isDirectory()) {
                    walk(file, function(err, res) {
                        results = results.concat(res);
                        if (!--pending) done(null, results);
                    });
                } else {
                    results.push(file);
                    if (!--pending) done(null, results);
                }
            });
        });
    });
}


/**
 * Walk a directory in a serial loop.
 * http://stackoverflow.com/questions/5827612/node-js-fs-readdir-recursive-directory-search
 *
 * Usage:
 * > walk('./path/to/directory', callbackFunction);
 *
 */
function walkSerial(dir, done) {
    var results = [];
    fs.readdir(dir, function(err, list) {
    if (err) return done(err);
        var i = 0;
        (function next() {
            var file = list[i++];
            if (!file) return done(null, results);
                file = dir + '/' + file;
                fs.stat(file, function(err, stat) {
                if (stat && stat.isDirectory()) {
                    walk(file, function(err, res) {
                        results = results.concat(res);
                        next();
                    });
                } else {
                    results.push(file);
                    next();
                }
            });
        })();
    });
}


/**
 * Wrapper for the UNIX find function. Possibly faster than the walk()
 * function above, particularly when multiple similar find commands are called
 * due to the find tool's built-in caching.
 * http://stackoverflow.com/questions/5827612/node-js-fs-readdir-recursive-directory-search
 *
 * Usage:
 * > find('./path/to/directory', callbackFunction);
 *
 */
function find(path, callback) {
    execFile('find', [path], function(err, stdout, stderr) {
        var fileList = stdout.split('\n');
        callback(err, fileList);
    });
}


exports.walk = walk;
exports.find = find;
